#ifndef _EXPORT_H_
#define _EXPORT_H_

/*
 * The opaque mutex type. A user which supplies a mutex implementation can store
 * any thread model specific data there.
 */
typedef struct _curl_mutex curl_mutex_t;

/*
 * Now, the typedefs of the required mutex function prototypes.
 */
typedef curl_mutex_t*	(*mutex_new_function)(void);
typedef void 		(*mutex_free_function)(curl_mutex_t*);
typedef void 		(*mutex_lock_function)(curl_mutex_t*);
typedef void 		(*mutex_unlock_function)(curl_mutex_t*);

/*
 * We use a struct of function pointers to gather all relevant
 * mutex functions in one place and call this struct:
 * curl_mutex_functions_t
 */
struct _curl_mutex_functions {
  mutex_new_function 	new_function;
  mutex_free_function 	free_function;
  mutex_lock_function 	lock_function;
  mutex_unlock_function	unlock_function;
};
typedef struct _curl_mutex_functions curl_mutex_functions_t;


/*
 * Last but not least the functions to register, retrieve and
 * unregister the mutex function struct (curl_mutex_functions_t)
 * with libcURL. Registering would typically be done at the start
 * of a user progarm.
 */
void 			curl_register_mutex_functions(curl_mutex_functions_t*);
curl_mutex_functions_t*	curl_get_mutex_functions(void);
void 			curl_unregister_mutex_functions(void);



/*
 * The next two function are not necessary  if we do not include the
 * mutex implementation for POSIX and WIN32 threads (syncimpl.c) in
 * libcURL.
 */


/*
 * Call this function to register mutex implementations with libcURL.
 *
 * Returns:
 *   -1 if your thread implementation is not supported
 *    1 if a out of memory situation occured
 *    0 if everything is ok
 *
 *  If your thread model is not supported by libcURL you have to provide
 *  your own implementation by the use of curl_register_mutex_functions(...).
 *  You could use syncimpl.c as a template. Look at sync.h, too.
 */
int curl_try_register_mutex_functions(void);

/*
 * This function removes the previously registered mutex functions
 * and frees some resources.
 */
void curl_try_unregister_mutex_functions(void);


#endif
